
var couchdb = 'http://localhost:5984/';
var database = 'uniprot';

function wsurl(request) {
  return couchdb + database + request;
};

var skip = 0;
var page = 10;
var sort = [{"_id": "asc"}]
var term = "";

function keypress(ele,event) {
   term = ele.value;
   skiptozero();
   return true;
};

function change(ele) {
	term = ele.value;
	skiptozero();
}

function addtoskip(value) {
   skip += value;
   renderproteins();
}

function skiptozero() {
   skip =  0;
   renderproteins();
}

function getaccession(doc,withlink=true){
	var accession = "";
	if (withlink) {
		accession += "<A href=\"#\" onclick=\"renderprotein(this.text);\">";
	}
	accession += doc.accession;
	if (withlink) {
		accession += "</A></td>";
	}
	return accession;
}

function getid(doc){
	return doc.id;
}

function getgene(doc){
	var gene = "";
	if (doc.gene) {
	    for (var j = 0; j < doc.gene.length; j++) {
			if (doc.gene[j] && doc.gene[j].name) {
				gene += (doc.gene[j].name.value + " ");
			}
		}
	}
	return gene;
}

function getdesc(doc) {
	var desc = "";
	if (doc.protein) {
		if (doc.protein.recommendedName) {
      if (doc.protein.recommendedName.fullName) {
				desc = doc.protein.recommendedName.fullName.value;
		  }
		}
	  if (desc == "" && doc.protein.submittedName) {
		  for (var j = 0; j < doc.protein.submittedName.length; j++) {
		    if (doc.protein.submittedName[j].fullName) {
				  desc = doc.protein.submittedName[j].fullName.value;
				  break;
			  }
		  }
		}
	}
	return desc;
}

function getorg(doc) {
	var org = "";
	if (doc.organism) {
		var taxa = "";
		if (doc.organism.taxonomy) {
			taxa = " (" + doc.organism.taxonomy + ")";
		}
		if (doc.organism.names) {
			for (var j = 0; j < doc.organism.names.length; j++) {
				if (doc.organism.names[j].type == "scientific") {
					org = doc.organism.names[j].value + taxa;
					break;
				}
			}
		}
    }	  
	return org;
}

function getseq(doc) {
	var seq = doc.sequence.sequence;
	var outseq = ""
	for (var i=0; i<seq.length; i+=60) {
	  outseq += seq.substring(i,i+60);
	  outseq += "<br/>";
    };
	return outseq;
}

function getgorefs(doc) {
	var dbrefs = doc.dbReferences;
	var gorefs = [];
	for (var i=0; i < dbrefs.length; i++) {
	  if (dbrefs[i]["type"] == "GO") {
		  var id = dbrefs[i]["id"]
			var term = dbrefs[i]["properties"]["term"];
			var type = term[0];
			term = term.substr(2);
			if (type == "P") {
			  type = "Biological process";
			} else if (type == "C") {
				type = "Cellular location";
			} else if (type == "F") {
				type = "Molecular function";
			}
			gorefs.push([id,term,type]);
		}
	}
	return gorefs;
}

function sorttable(header) {
	var thekey = "_id";
	if (header.innerHTML == "Accession") {
	    thekey = "accession";
	} else if (header.innerHTML == "Gene") {
	    thekey = "gene.0.name.value";
	} else if (header.innerHTML == "Organism") {
		thekey = "organism.names.0.value";
	} 
	if (sort.length == 0 || sort[0][thekey] != "asc") {
		sort = [{[thekey]: "asc"}];
	} else {
		sort = [{[thekey]: "desc"}];
	}
	skiptozero();
}

function renderproteins() {
  var criteria = {"selector": {}, "sort": sort, "limit": (page+1), "skip": skip};
  if (term != "") {
	var caseinsensitiveterm = "(?i)"+term;
	criteria["selector"] = {"$or": [{"accession": {"$regex": caseinsensitiveterm}},
	                                {"gene": {"$elemMatch": {"name.value": {"$regex": caseinsensitiveterm}}}},
	                {"protein.recommendedName.fullName.value": {"$regex": caseinsensitiveterm}},								
									{"protein.submittedName": {"$elemMatch": {"fullName.value": {"$regex": caseinsensitiveterm}}}},
									{"organism.names": {"$elemMatch": {"value": {"$regex": caseinsensitiveterm}}}},
									{"organism.taxonomy": {"$eq": parseInt(term)}}
								   ]};
  }
  json_ws_post(wsurl('/_find'),criteria,function(data) {
    var table = '';
	table += '<table width="100%">';
	table += '<tr>';
	table += '<th width="50px;" onclick="sorttable(this);">Index</th>';
	table += '<th width="120px;" onclick="sorttable(this);">Accession</th>';
	//table += '<th>ID</th>';
	table += '<th width="100px;" onclick="sorttable(this);">Gene</th>';
	table += '<th>Description</th>';
	table += '<th width="200px;" onclick="sorttable(this);">Organism</th>';
	table += '</tr>';
	for (var i = 0; i < Math.min(data.docs.length,page); i++ ) {
	  table += "<tr valign=\"top\">";
	  table += "<td width=\"50px;\">";
	  table += i+1+skip;
	  table += "</td>";
	  table += "<td width=\"120px;\">";
	  table += getaccession(data.docs[i],true);
	  table += "</td>";
	  //table += "<td>";
	  //table += getid(data.docs[i]);
	  //table += "</td>";
	  table += "<td width=\"100px;\">";
	  table += getgene(data.docs[i]);
	  table += "</td>";
	  table += "<td>";
	  table += getdesc(data.docs[i]);
	  table += "</td>";
	   table += "<td width=\"250px;\">";
	  table += getorg(data.docs[i]);
	  table += "</td>";
	  table += "</tr>";
	}
	table += "</table>";
	table += "<div style=\"float:right;\"><P>";
	var needsep = false;
	if (skip > 0) {
	  table += "<A href=\"#\" onclick=\"skiptozero();\">Beginning</A>"; 
	  table += " - <A href=\"#\" onclick=\"addtoskip(-page);\">Previous</A>";
	  needsep = true;
	}
	if (data.docs.length > page) {
	  if (needsep) {
		table += " - ";
	  }
	  table += "<A href=\"#\" onclick=\"addtoskip(page);\">Next</A>";
   	}
	table += "</P></div>"
    document.getElementById('container').innerHTML = table;
	document.getElementById('search').className = "show";
  });
  return true;
}

function renderprotein(accession) {
  var criteria = {"selector": {"accession": accession}, "limit": 1};
  json_ws_post(wsurl('/_find'),criteria,function(data) {
	//console.log(data);
  var table = "<table>";
	table += "<tr><th>Accession</th><td>";
	table += getaccession(data.docs[0],false);
	table += "</td></tr>"; 
	table += "<tr><th>ID</th><td>";
	table += getid(data.docs[0]);
	table += "</td></tr>"; 
	table += "<tr><th>Gene</th><td>";
	table += getgene(data.docs[0]); 
  table += "</td></tr>"
	table += "<tr><th>Description</th><td>";
	table += getdesc(data.docs[0]);
	table += "</td></tr>";
	table += "<tr><th>Organism</th><td>";
	table += getorg(data.docs[0]);
	table += "</td></tr>"; 
	table += "<tr><th valign=\"top\">Sequence</th><td class=\"sequence\">";
	table += getseq(data.docs[0]);
	table += "</td></tr>"; 
	table += "</table>";
	var gorefs = getgorefs(data.docs[0]);
	if (gorefs.length > 0) {
	  table += "<p/>";
	  table += "<table>";
	  table += "<h2>Gene Ontology Annotations</h2>";
	  table += "<tr><th>Accession</th><th>Term</th><th>Type</th></tr>";
	  for (var i=0; i<gorefs.length; i++) {
			table += "<tr>";
		  table += "<td>"+gorefs[i][0]+"</td>";
			table += "<td>"+gorefs[i][1]+"</td>";
			table += "<td>"+gorefs[i][2]+"</td>";
			table += "</tr>";
	  }
	  table += "</table>";
	};
	table += "<P align=\"right\"><A href=\"#\" onclick=\"renderproteins();\">Return</A></P>"
  document.getElementById('container').innerHTML = table;
	document.getElementById('search').className = "hide";
  });
}


